#include <Encoder.h> //From PJRC. See notes here: https://www.pjrc.com/teensy/td_libs_Encoder.html

//According to forum, for Nano33 IOT:
//pins 2, 3, 9, 10, and 11 work as interrupt pin

volatile byte encoderButtonChange = false; //Variables that may change in an interrupt are declared volatile

byte greenLED = 4;  //pin
byte redLED = 5;    //pin

//structure to hold info related to pin timing. 
//Next level would be: make object to include update methods...
struct timer {
  unsigned long last;
  unsigned long interval;
  bool state;
  int pin;
};
timer red, green;

//A rotary encoder can signal clockwise and counterclockwise relative movement.
//Mine uses two timing pins and a ground - comparing the rising and falling signals of 
//the timing pin indicate which direction and how much it has turned. 
//Most also include a button which is essentially its own separate circuit with 2 pins.
Encoder knob(2, 3); //the Encoder library is not used for the button, but is for measure the rotation. Internally, it will attach interrupt handlers to the signal pins (if supported)
long position = 0; //we'll keep track of changes in a position variable

void setup() {
  pinMode(9, INPUT_PULLUP); //no internal resistor needed, but by default input will be "HIGH"/pulled up
  pinMode(LED_BUILTIN, OUTPUT);

  pinMode(greenLED, OUTPUT);
  pinMode(redLED, OUTPUT);
  digitalWrite(greenLED, HIGH);
  digitalWrite(redLED, HIGH);

  //user the timer struct to store all the parameters for two different flashing leds
  red = {0, 1000, LOW, redLED};
  green = {0, 300, HIGH, greenLED};

  //While the encoder library uses interrupts internally, we can add our own, to.
  //Set the microcontroller to runn the "handleInterrupt" function whenever pin 9 changes
  attachInterrupt(digitalPinToInterrupt(9), handleInterrupt, CHANGE);

  //And some serial
  Serial.begin(9600);
  digitalWrite(LED_BUILTIN, HIGH);
}

void loop() {
  updateLEDTimers();
  if (encoderButtonChange) handleEncoderButton();
  handleKnob();
  //delay(1);
}

//check if the encoder knob has moved
void handleKnob() {
  long newPos = knob.read(); //read the position
  if (newPos != position) { //compare, update if changed
    position = newPos;
    Serial.print("Encoder: ");
    Serial.println(position);
    red.interval = position*2;
    green.interval = position/1.5;
  }
}

//Funtion to toggle state for an LED pin if an interval has passed.
//Since it's repeating the same code for red and green, this would
//be better written as a function or as a method for a timer class.
void updateLEDTimers() {
  unsigned long nowish = millis();
  if (nowish - red.last >= red.interval) {
    red.state = !red.state; //flip state
    red.last = nowish;
    digitalWrite(redLED, red.state);
  }
  
  nowish = millis();
  if (nowish - green.last >= green.interval) {
    green.state = !green.state; //flip state
    green.last = nowish;
    digitalWrite(greenLED, green.state);
  }
}

//This is called from the main loop if the interrupt flag is true
void handleEncoderButton() {
  if (!digitalRead(9)) { //pressed - "INPUT_PULLUP" pin goes low when pressed
    digitalWrite(LED_BUILTIN, HIGH);
    red.interval = 99;
    green.interval = 66;
  } else { //released
    digitalWrite(LED_BUILTIN, LOW);
    red.interval = 1000;
    green.interval = 300;    
  }
  encoderButtonChange = false; //change handled, clear flag
}

//The handler only "sets a flag", which will be seen and processed in the main loop.
//Data type for the flag is volatile, indicating it may change in an interrupt
void handleInterrupt() {
  encoderButtonChange = true;  
}
